@@ -22,7 +22,6 @@
 # ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
-
 from collections import namedtuple
 import traceback
 import sys
@@ -30,10 +29,12 @@ import os
 import imp
 import pkgutil
 import time
+import threading
 
-from util import *
-from i18n import _
-from util import profiler, PrintError, DaemonThread, UserCancelled
+from .util import print_error
+from .i18n import _
+from .util import profiler, PrintError, DaemonThread, UserCancelled, ThreadJob
+from . import bitcoin
 
 plugin_loaders = {}
 hook_names = set()
@@ -93,7 +94,7 @@ class Plugins(DaemonThread):
 
     def load_plugin(self, name):
         if name in self.plugins:
-            return
+            return self.plugins[name]
         full_name = 'electrum_plugins.' + name + '.' + self.gui_name
         loader = pkgutil.find_loader(full_name)
         if not loader:
@@ -156,7 +157,7 @@ class Plugins(DaemonThread):
         return out
 
     def register_wallet_type(self, name, gui_good, wallet_type):
-        from wallet import register_wallet_type, register_constructor
+        from .wallet import register_wallet_type, register_constructor
         self.print_error("registering wallet type", (wallet_type, name))
         def loader():
             plugin = self.get_plugin(name)
@@ -165,7 +166,7 @@ class Plugins(DaemonThread):
         plugin_loaders[wallet_type] = loader
 
     def register_keystore(self, name, gui_good, details):
-        from keystore import register_keystore
+        from .keystore import register_keystore
         def dynamic_constructor(d):
             return self.get_plugin(name).keystore_class(d)
         if details[0] == 'hardware':
@@ -186,7 +187,7 @@ class Plugins(DaemonThread):
 
 
 def hook(func):
-    hook_names.add(func.func_name)
+    hook_names.add(func.__name__)
     return func
 
 def run_hook(name, *args):
@@ -253,6 +254,9 @@ class BasePlugin(PrintError):
     def is_available(self):
         return True
 
+    def can_user_disable(self):
+        return True
+
     def settings_dialog(self):
         pass
 
@@ -263,7 +267,7 @@ class DeviceNotFoundError(Exception):
 class DeviceUnpairableError(Exception):
     pass
 
-Device = namedtuple("Device", "path interface_number id_ product_key")
+Device = namedtuple("Device", "path interface_number id_ product_key usage_page")
 DeviceInfo = namedtuple("DeviceInfo", "device label initialized")
 
 class DeviceMgr(ThreadJob, PrintError):
@@ -299,7 +303,7 @@ class DeviceMgr(ThreadJob, PrintError):
 
     def __init__(self, config):
         super(DeviceMgr, self).__init__()
-        # Keyed by xpub.  The value is the device id 
+        # Keyed by xpub.  The value is the device id
         # has been paired, and None otherwise.
         self.xpub_ids = {}
         # A list of clients.  The key is the client, the value is
@@ -308,6 +312,8 @@ class DeviceMgr(ThreadJob, PrintError):
         # What we recognise.  Each entry is a (vendor_id, product_id)
         # pair.
         self.recognised_hardware = set()
+        # Custom enumerate functions for devices we don't know about.
+        self.enumerate_func = set()
         # For synchronization
         self.lock = threading.RLock()
         self.hid_lock = threading.RLock()
@@ -330,6 +336,9 @@ class DeviceMgr(ThreadJob, PrintError):
         for pair in device_pairs:
             self.recognised_hardware.add(pair)
 
+    def register_enumerate_func(self, func):
+        self.enumerate_func.add(func)
+
     def create_client(self, device, handler, plugin):
         # Get from cache first
         client = self.client_lookup(device.id_)
@@ -358,15 +367,20 @@ class DeviceMgr(ThreadJob, PrintError):
             if not xpub in self.xpub_ids:
                 return
             _id = self.xpub_ids.pop(xpub)
-        client = self.client_lookup(_id)
-        self.clients.pop(client, None)
-        if client:
-            client.close()
+            self._close_client(_id)
 
     def unpair_id(self, id_):
         xpub = self.xpub_by_id(id_)
         if xpub:
             self.unpair_xpub(xpub)
+        else:
+            self._close_client(id_)
+
+    def _close_client(self, id_):
+        client = self.client_lookup(id_)
+        self.clients.pop(client, None)
+        if client:
+            client.close()
 
     def pair_xpub(self, xpub, id_):
         with self.lock:
@@ -388,6 +402,8 @@ class DeviceMgr(ThreadJob, PrintError):
 
     def client_for_keystore(self, plugin, handler, keystore, force_pair):
         self.print_error("getting client for keystore")
+        if handler is None:
+            raise BaseException(_("Handler not found for") + ' ' + plugin.name + '\n' + _("A library is probably missing."))
         handler.update_status(False)
         devices = self.scan_devices()
         xpub = keystore.xpub
@@ -418,14 +434,14 @@ class DeviceMgr(ThreadJob, PrintError):
     def force_pair_xpub(self, plugin, handler, info, xpub, derivation, devices):
         # The wallet has not been previously paired, so let the user
         # choose an unpaired device and compare its first address.
-
+        xtype = bitcoin.xpub_type(xpub)
         client = self.client_lookup(info.device.id_)
         if client and client.is_pairable():
             # See comment above for same code
             client.handler = handler
             # This will trigger a PIN/passphrase entry request
             try:
-                client_xpub = client.get_xpub(derivation)
+                client_xpub = client.get_xpub(derivation, xtype)
             except (UserCancelled, RuntimeError):
                  # Bad / cancelled PIN / passphrase
                 client_xpub = None
@@ -436,11 +452,11 @@ class DeviceMgr(ThreadJob, PrintError):
         # The user input has wrong PIN or passphrase, or cancelled input,
         # or it is not pairable
         raise DeviceUnpairableError(
-            _('Electrum cannot pair with your %s.\n\n'
+            _('Electrum cannot pair with your {}.\n\n'
               'Before you request bitcoins to be sent to addresses in this '
               'wallet, ensure you can pair with your device, or that you have '
               'its seed (and passphrase, if any).  Otherwise all bitcoins you '
-              'receive will be unspendable.') % plugin.device)
+              'receive will be unspendable.').format(plugin.device))
 
     def unpaired_device_infos(self, handler, plugin, devices=None):
         '''Returns a list of DeviceInfo objects: one for each connected,
@@ -466,9 +482,9 @@ class DeviceMgr(ThreadJob, PrintError):
             infos = self.unpaired_device_infos(handler, plugin, devices)
             if infos:
                 break
-            msg = _('Could not connect to your %s.  Verify the cable is '
+            msg = _('Please insert your {}.  Verify the cable is '
                     'connected and that no other application is using it.\n\n'
-                    'Try to connect again?') % plugin.device
+                    'Try to connect again?').format(plugin.device)
             if not handler.yes_no_question(msg):
                 raise UserCancelled()
             devices = None
@@ -478,7 +494,7 @@ class DeviceMgr(ThreadJob, PrintError):
         for info in infos:
             if info.label == keystore.label:
                 return info
-        msg = _("Please select which %s device to use:") % plugin.device
+        msg = _("Please select which {} device to use:").format(plugin.device)
         descriptions = [info.label + ' (%s)'%(_("initialized") if info.initialized else _("wiped")) for info in infos]
         c = handler.query_choice(msg, descriptions)
         if c is None:
@@ -486,7 +502,8 @@ class DeviceMgr(ThreadJob, PrintError):
         info = infos[c]
         # save new label
         keystore.set_label(info.label)
-        handler.win.wallet.save_keystore()
+        if handler.win.wallet is not None:
+            handler.win.wallet.save_keystore()
         return info
 
     def scan_devices(self):
@@ -498,18 +515,25 @@ class DeviceMgr(ThreadJob, PrintError):
         self.print_error("scanning devices...")
         with self.hid_lock:
             hid_list = hid.enumerate(0, 0)
+
         # First see what's connected that we know about
         devices = []
         for d in hid_list:
             product_key = (d['vendor_id'], d['product_id'])
             if product_key in self.recognised_hardware:
                 # Older versions of hid don't provide interface_number
-                interface_number = d.get('interface_number', 0)
-                serial = d['serial_number']
-                if len(serial) == 0:
-                    serial = d['path']
+                interface_number = d.get('interface_number', -1)
+                usage_page = d['usage_page']
+                id_ = d['serial_number']
+                if len(id_) == 0:
+                    id_ = str(d['path'])
+                id_ += str(interface_number) + str(usage_page)
                 devices.append(Device(d['path'], interface_number,
-                                      serial, product_key))
+                                      id_, product_key, usage_page))
+
+        # Let plugin handlers enumerate devices we don't know about
+        for f in self.enumerate_func:
+            devices.extend(f())
 
         # Now find out what was disconnected
         pairs = [(dev.path, dev.id_) for dev in devices]
